@@ -1,5 +1,7 @@
 package Bio::DB::Sam;
-# $Id: Sam.pm 22900 2010-03-25 16:25:53Z lstein $
+# $Id: Sam.pm 23058 2010-04-18 21:29:29Z lstein $
+
+our $VERSION = '1.16';
 
 =head1 NAME
 
@@ -102,8 +104,9 @@ genome sequence, and the BAM file that contains the query sequences
 and their alignments. If either of the two files needs to be indexed,
 the indexing will happen automatically. You can then query the
 database for alignment features by combinations of name, position,
-type, and feature tag. The high-level API provides access to up to
-four feature "types":
+type, and feature tag.
+
+The high-level API provides access to up to four feature "types":
 
  * "match": The "raw" unpaired alignment between a read and the
    reference sequence.
@@ -149,6 +152,22 @@ callback that will be invoked on every column of the alignment for the
 purpose of calculating coverage, quality score metrics, or SNP
 calling.
 
+B<Access to the reference sequence> When you create the Bio::DB::Sam
+object, you can pass the path to a FASTA file containing the reference
+sequence. Alternatively, you may pass an object that knows how to
+retrieve DNA sequences across a range via the seq() of fetch_seq()
+methods, as described under new().
+
+If the SAM/BAM file has MD tags, then these tags will be used to
+reconstruct the reference sequence when necessary, in which case you
+can completely omit the -fasta argument. Note that not all SAM/BAM
+files have MD tags, and those that do may not use them correctly due
+to the newness of this part of the SAM spec. You may wish to populate
+these tags using samtools' "calmd" command.
+
+If the -fasta argument is omitted and no MD tags are present, then the
+reference sequence will be returned as 'N'.
+
 The B<main object classes> that you will be dealing with in the
 high-level API are as follows:
 
@@ -237,6 +256,10 @@ follows:
 
   -split          The same as -split_splices.
 
+  -autoindex      Create a BAM index file if one does not exist
+                   or the current one has a modification date
+                   earlier than the BAM file.
+
 An example of a typical new() constructor invocation is:
  
   $sam = Bio::DB::Sam->new(-fasta => '/home/projects/genomes/hu17.fa',
@@ -297,6 +320,11 @@ B<subsequently.>
 Return the Bio::DB::Bam::Header object associated with the BAM
 file. You can manipulate the header using the low-level API.
 
+=item $bam    = $sam->bam
+
+Returns the low-level Bio::DB::Bam object associated with the opened
+file.
+
 =item $fai    = $sam->fai
 
 Returns the Bio::DB::Sam::Fai object associated with the Fasta
@@ -1023,6 +1051,14 @@ space for the operation or the process will be terminated in the C
 library layer. The result code is currently always zero, but in the
 future may return a negative value to indicate failure.
 
+=item $index = Bio::DB::Bam->index('/path/to/file.bam',$reindex)
+
+Attempt to open the index for the indicated BAM file. If $reindex is
+true, and the index either does not exist or is out of date with
+respect to the BAM file (by checking modification dates), then attempt
+to rebuild the index. Will throw an exception if the index does not
+exist or if attempting to rebuild the index was unsuccessful.
+
 =item $index = Bio::DB::Bam->index_open('/path/to/file.bam')
 
 Attempt to open the index file for a BAM file, returning a
@@ -1267,7 +1303,6 @@ use Bio::SeqFeature::Lite;
 use Bio::PrimarySeq;
 
 use base 'DynaLoader';
-our $VERSION = '1.14';
 bootstrap Bio::DB::Sam;
 
 use Bio::DB::Bam::Alignment;
@@ -1279,11 +1314,12 @@ use Bio::DB::Bam::ReadIterator;
 
 sub new {
     my $class         = shift;
-    my %args          = @_;
+    my %args          = $_[0] =~ /^-/ ? @_ : (-bam=>shift);
     my $bam_path      = $args{-bam}   or croak "-bam argument required";
     my $fa_path       = $args{-fasta};
     my $expand_flags  = $args{-expand_flags};
     my $split_splices = $args{-split} || $args{-split_splices};
+    my $autoindex     = $args{-autoindex};
 
     # file existence checks
     unless ($class->is_remote($bam_path)) {
@@ -1302,12 +1338,15 @@ sub new {
 	fa_path       => $fa_path,
 	expand_flags  => $expand_flags,
 	split_splices => $split_splices,
+	autoindex     => $autoindex,
     },ref $class || $class;
     $self->header;  # catch it
 
     return $self;
 }
 
+sub bam { shift->{bam} }
+
 sub is_remote {
     my $self = shift;
     my $path = shift;
@@ -1379,6 +1418,13 @@ sub split_splices {
     $d;
 }
 
+sub autoindex {
+    my $self = shift;
+    my $d    = $self->{autoindex};
+    $self->{autoindex} = shift if @_;
+    $d;
+}
+
 sub reset_read {
     my $self = shift;
     $self->{bam}->header;
@@ -1660,7 +1706,7 @@ sub features {
 	    $self->reset_read;
 	    my $code = eval "sub {my \$a=shift;$filter;1}";
 	    die $@ if $@;
-	    return Bio::DB::Bam::ReadIterator->new($self->{bam},$code);
+	    return Bio::DB::Bam::ReadIterator->new($self,$self->{bam},$code);
 	}
 
 	# TAM filehandle retrieval is requested
@@ -1887,7 +1933,7 @@ sub _segment_search {
 
 sub bam_index {
     my $self = shift;
-    return Bio::DB::Bam->index($self->{bam_path});
+    return $self->{bai} ||= Bio::DB::Bam->index($self->{bam_path},$self->autoindex);
 }
 
 sub _features_fh {
@@ -2012,43 +2058,55 @@ package Bio::DB::Bam;
 
 use File::Spec;
 use Cwd;
+use Carp 'croak';
 
 sub index {
     my $self = shift;
     my $path = shift;
+    my $autoindex = shift;
 
     return $self->index_open_in_safewd($path) if Bio::DB::Sam->is_remote($path);
 
-    unless (-e "${path}.bai" && (-M $path >= -M "${path}.bai")) {
-	# if bam file is not sorted, then index_build will exit.
-	# we spawn a shell to intercept this eventuality
-	print STDERR "[bam_index_build] creating index for $path\n" if -t STDOUT;
-
-	my $result = open my $fh,"-|";
-	die "Couldn't fork $!" unless defined $result;
-
-	if ($result == 0) { # in child
-	    # dup stderr to stdout so that we can intercept messages from library
-	    open STDERR,">&STDOUT";  
-	    $self->index_build($path);
-	    exit 0;
-	}
-
-	my $mesg = <$fh>;
-	$mesg  ||= '';
-	close $fh;
-	if ($mesg =~ /not sorted/i) {
-	    print STDERR "[bam_index_build] sorting by coordinate...\n" if -t STDOUT;
-	    $self->sort_core(0,$path,"$path.sorted");
-	    rename "$path.sorted.bam",$path;
-	    $self->index_build($path);
-	} elsif ($mesg) {
-	    die $mesg;
-	}
+    if ($autoindex) {
+	$self->reindex($path) unless
+	    -e "${path}.bai" && mtime($path) <= mtime("${path}.bai");
     }
+
+    croak "No index file for $path; try opening file with -autoindex" unless -e "${path}.bai";
     return $self->index_open($path);
 }
 
+sub reindex {
+    my $self = shift;
+    my $path = shift;
+
+    # if bam file is not sorted, then index_build will exit.
+    # we spawn a shell to intercept this eventuality
+    print STDERR "[bam_index_build] creating index for $path\n" if -t STDOUT;
+
+    my $result = open my $fh,"-|";
+    die "Couldn't fork $!" unless defined $result;
+
+    if ($result == 0) { # in child
+	# dup stderr to stdout so that we can intercept messages from library
+	open STDERR,">&STDOUT";  
+	$self->index_build($path);
+	exit 0;
+    }
+
+    my $mesg = <$fh>;
+    $mesg  ||= '';
+    close $fh;
+    if ($mesg =~ /not sorted/i) {
+	print STDERR "[bam_index_build] sorting by coordinate...\n" if -t STDOUT;
+	$self->sort_core(0,$path,"$path.sorted");
+	rename "$path.sorted.bam",$path;
+	$self->index_build($path);
+    } elsif ($mesg) {
+	die $mesg;
+    }
+}
+
 # same as index_open(), but changes current wd to TMPDIR to accomodate
 # the C library when it tries to download the index file from remote
 # locations.
@@ -2062,6 +2120,11 @@ sub index_open_in_safewd {
     $result;
 }
 
+sub mtime {
+    my $path = shift;
+    (stat($path))[9];
+}
+
 
 1;
 __END__
